@@ -9,7 +9,7 @@
 
 /* .Call calls */
 extern SEXP _BHSBVAR_MAIN(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-extern SEXP _BHSBVAR_post_A_max(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP _BHSBVAR_post_A_optim(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _BHSBVAR_prior_nonc_t(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _BHSBVAR_prior_t(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _BHSBVAR_prior_t_n(SEXP, SEXP, SEXP, SEXP);
@@ -17,7 +17,7 @@ extern SEXP _BHSBVAR_prior_t_p(SEXP, SEXP, SEXP, SEXP);
 
 static const R_CallMethodDef CallEntries[] = {
     {"_BHSBVAR_MAIN",         (DL_FUNC) &_BHSBVAR_MAIN,         23},
-    {"_BHSBVAR_post_A_max",   (DL_FUNC) &_BHSBVAR_post_A_max,   13},
+    {"_BHSBVAR_post_A_optim", (DL_FUNC) &_BHSBVAR_post_A_optim, 13},
     {"_BHSBVAR_prior_nonc_t", (DL_FUNC) &_BHSBVAR_prior_nonc_t,  5},
     {"_BHSBVAR_prior_t",      (DL_FUNC) &_BHSBVAR_prior_t,       4},
     {"_BHSBVAR_prior_t_n",    (DL_FUNC) &_BHSBVAR_prior_t_n,     4},
